/* --------------------------------------------------------------------------------------------
 * Copyright (c) Remy Suen. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const docker_1 = require("./docker");
const dockerfileParser_1 = require("./parser/dockerfileParser");
class DockerDefinition {
    computeBuildStageDefinition(uri, dockerfile, position) {
        let source = undefined;
        for (let instruction of dockerfile.getCOPYs()) {
            let range = instruction.getFromValueRange();
            if (range && range.start.line === position.line && range.start.character <= position.character && position.character <= range.end.character) {
                source = instruction.getFromValue();
                break;
            }
        }
        for (let instruction of dockerfile.getFROMs()) {
            let range = instruction.getBuildStageRange();
            if (range &&
                ((range.start.line === position.line && range.start.character <= position.character && position.character <= range.end.character) || (instruction.getBuildStage() === source))) {
                return vscode_languageserver_1.Location.create(uri, range);
            }
        }
        return null;
    }
    static computeVariableDefinition(dockerfile, position) {
        let variable = null;
        let variables = {};
        for (let arg of dockerfile.getARGs()) {
            let property = arg.getProperty();
            // might be an ARG with no arguments
            if (property) {
                // is the caret inside the definition itself
                if (docker_1.Util.isInsideRange(position, property.getNameRange())) {
                    return property;
                }
                variables[property.getName()] = property;
            }
        }
        for (let env of dockerfile.getENVs()) {
            let properties = env.getProperties();
            for (let property of properties) {
                // is the caret inside the definition itself
                if (docker_1.Util.isInsideRange(position, property.getNameRange())) {
                    return variables[property.getName()] ? variables[property.getName()] : property;
                }
                else if (!variables[property.getName()]) {
                    variables[property.getName()] = property;
                }
            }
        }
        for (let instruction of dockerfile.getInstructions()) {
            for (let variable of instruction.getVariables()) {
                if (docker_1.Util.isInsideRange(position, variable.getNameRange())) {
                    if (variables[variable.getName()]) {
                        return variables[variable.getName()];
                    }
                    return null;
                }
            }
        }
        return null;
    }
    computeVariableDefinition(uri, dockerfile, position) {
        let property = DockerDefinition.computeVariableDefinition(dockerfile, position);
        return property ? vscode_languageserver_1.Location.create(uri, property.getNameRange()) : null;
    }
    computeDefinition(document, position) {
        let parser = new dockerfileParser_1.DockerfileParser();
        let dockerfile = parser.parse(document);
        let definition = this.computeBuildStageDefinition(document.uri, dockerfile, position);
        if (definition !== null) {
            return definition;
        }
        definition = this.computeVariableDefinition(document.uri, dockerfile, position);
        if (definition !== null) {
            return definition;
        }
        return null;
    }
}
exports.DockerDefinition = DockerDefinition;
