"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Remy Suen. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
const vscode_languageserver_1 = require("vscode-languageserver");
const docker_1 = require("../docker");
class Property {
    constructor(document, escapeChar, arg, arg2) {
        this.name = null;
        this.value = null;
        if (arg2) {
            this.nameRange = arg.getRange();
            this.name = document.getText().substring(document.offsetAt(this.nameRange.start), document.offsetAt(this.nameRange.end));
            let valueRange = arg2.getRange();
            let value = document.getText().substring(document.offsetAt(valueRange.start), document.offsetAt(valueRange.end));
            this.value = Property.getValue(value, escapeChar);
            this.range = vscode_languageserver_1.Range.create(this.nameRange.start, valueRange.end);
        }
        else {
            this.nameRange = Property.getNameRange(document, arg);
            this.name = document.getText().substring(document.offsetAt(this.nameRange.start), document.offsetAt(this.nameRange.end));
            let valueRange = Property.getValueRange(document, escapeChar, arg);
            if (valueRange) {
                let value = document.getText().substring(document.offsetAt(valueRange.start), document.offsetAt(valueRange.end));
                this.value = Property.getValue(value, escapeChar);
            }
            this.range = arg.getRange();
        }
    }
    getRange() {
        return this.range;
    }
    getName() {
        return this.name;
    }
    getNameRange() {
        return this.nameRange;
    }
    getValue() {
        return this.value;
    }
    static getNameRange(document, arg) {
        let index = arg.getValue().indexOf('=');
        if (index !== -1) {
            return vscode_languageserver_1.Range.create(arg.getRange().start, document.positionAt(document.offsetAt(arg.getRange().start) + index));
        }
        // no '=' found, just defined the ARG's name
        return arg.getRange();
    }
    static getValueRange(document, escapeChar, arg) {
        let argValue = arg.getValue();
        let index = argValue.indexOf('=');
        if (index === -1) {
            // no value declared if no '=' found
            return null;
        }
        // trim the leading whitespace of the variable's value
        let startIndex = index < argValue.length - 1 ? docker_1.Util.findLeadingNonWhitespace(argValue.substring(index + 1), escapeChar) : 0;
        return vscode_languageserver_1.Range.create(document.positionAt(document.offsetAt(arg.getRange().start) + index + startIndex + 1), document.positionAt(document.offsetAt(arg.getRange().end)));
    }
    /**
     * Returns the actual value of this ARG instruction's declared
     * variable. The value will have its escape characters removed if
     * applicable.
     *
     * @return the value that this ARG instruction's declared
     *         variable will resolve to, may be null if no value is
     *         defined, may be the empty string if the value only
     *         consists of whitespace
     */
    static getValue(value, escapeChar) {
        let literal = false;
        let first = value.charAt(0);
        let last = value.charAt(value.length - 1);
        if ((first === '"' && last === '"') || (first === '\'' && last === '\'')) {
            literal = true;
            value = value.substring(1, value.length - 1);
        }
        let escapedValue = "";
        parseValue: for (let i = 0; i < value.length; i++) {
            let char = value.charAt(i);
            if (char === escapeChar) {
                if (i + 1 === value.length) {
                    break;
                }
                char = value.charAt(i + 1);
                if (literal) {
                    if (char === '\n') {
                        i++;
                    }
                    else {
                        if (char === escapeChar) {
                            i++;
                        }
                        escapedValue = escapedValue + escapeChar;
                    }
                    continue parseValue;
                }
                else if (char === escapeChar) {
                    // double escape, append one and move on
                    escapedValue = escapedValue + escapeChar;
                    i++;
                }
                else if (char === '\r') {
                    if (value.charAt(i + 2) === '\n') {
                        i++;
                    }
                    i++;
                }
                else if (char === '\n') {
                    i++;
                }
                else if (char === ' ' || char === '\t') {
                    for (let j = i + 2; j < value.length; j++) {
                        let char2 = value.charAt(j);
                        if (char2 === ' ' || char2 === '\t') {
                            continue;
                        }
                        else if (char2 === '\r') {
                            if (value.charAt(j + 1) === '\n') {
                                i = j + 1;
                            }
                            else {
                                i = j;
                            }
                            continue parseValue;
                        }
                        else if (char2 === '\n') {
                            // the expectation is that this is === '\n'
                            i = j;
                            continue parseValue;
                        }
                        else if (char2 === escapeChar) {
                            // add the escaped character
                            escapedValue = escapedValue + char;
                            // now start parsing from the next escape character
                            i = i + 1;
                        }
                        else {
                            // the expectation is that this j = i + 2 here
                            escapedValue = escapedValue + char + char2;
                            i = j;
                            continue parseValue;
                        }
                    }
                }
                else {
                    // any other escapes are simply ignored
                    escapedValue = escapedValue + char;
                    i++;
                }
            }
            else {
                escapedValue = escapedValue + char;
            }
        }
        return escapedValue;
    }
}
exports.Property = Property;
